﻿namespace DynamicDecoratorDemo
{
    using System;
    using Patterns.Decorator;

    public interface IElement
    {
        string Text { get; set; }

        void Draw();
    }

    public class Element : IElement
    {
        public string Text { get; set; }

        public void Draw()
        {
            Console.WriteLine("Drawing element ({0})", this.Text);
        }
    }

    public class ElementStrikedDecorator : DynamicDecorator<IElement>
    {
        public ElementStrikedDecorator(IElement component) : base(component) { }

        public void Draw()
        {
            this._component.Draw();
            this.Strike();
        }

        private void Strike()
        {
            Console.WriteLine("Striked");
        }
    }

    public class ElementBgndDecorator : DynamicDecorator<IElement>
    {
        public ElementBgndDecorator(IElement component) : base(component) { }

        public string BackgroundFileName { get; set; }

        public void Draw()
        {
            this.SetBackground();
            this._component.Draw();
        }

        private void SetBackground()
        {
            Console.WriteLine("Background: {0}", this.BackgroundFileName);
        }
    }

    public class MyDynamic : System.Dynamic.IDynamicMetaObjectProvider
    {
        public void DefinedMethod()
        {
            Console.WriteLine("DefinedMethod");
        }

        public System.Dynamic.DynamicMetaObject GetMetaObject(System.Linq.Expressions.Expression parameter)
        {
            throw new NotImplementedException();
        }
    }

    class Program
    {
        public static void DrawElement(string title, IElement element)
        {
            Console.WriteLine("{0} ---------------------------", title);
            element.Draw();
            Console.WriteLine(string.Empty);
        }

        static void Main(string[] args)
        {

            //dynamic myDynamicObject = new MyDynamic();
            //myDynamicObject.DefinedMethod();
            //myDynamicObject.Test();

            Element element = new Element();
            DrawElement("Base element", element);

            dynamic elementBgnd = new ElementBgndDecorator(element);
            elementBgnd.BackgroundFileName = "SomeBgndFile.png";

            dynamic elementStriked = new ElementStrikedDecorator(elementBgnd.Interface);
            elementStriked.Text = "Demo";

            DrawElement("Striked element with background", elementStriked.Interface);

            IElement sourceElement = elementStriked.GetComponent();
            DrawElement("Element with background", sourceElement);

            Console.WriteLine("Press any key ...");
            Console.ReadKey();
        }
    }
}
